@@ -253,7 +253,7 @@ int CUDTUnited::cleanup()
    return 0;
 }
 
-UDTSOCKET CUDTUnited::newSocket(const int& af, const int& type)
+UDTSOCKET CUDTUnited::newSocket(int af, int type)
 {
    if ((type != SOCK_STREAM) && (type != SOCK_DGRAM))
       throw CUDTException(5, 3, 0);
@@ -301,6 +301,7 @@ UDTSOCKET CUDTUnited::newSocket(const int& af, const int& type)
    catch (...)
    {
       //failure and rollback
+      CGuard::leaveCS(m_ControlLock);
       delete ns;
       ns = NULL;
    }
@@ -436,7 +437,7 @@ int CUDTUnited::newConnection(const UDTSOCKET listen, const sockaddr* peer, CHan
    CGuard::leaveCS(ls->m_AcceptLock);
 
    // acknowledge users waiting for new connections on the listening socket
-   m_EPoll.enable_read(listen, ls->m_pUDT->m_sPollID);
+   m_EPoll.update_events(listen, ls->m_pUDT->m_sPollID, UDT_EPOLL_IN, true);
 
    CTimer::triggerEvent();
 
@@ -496,7 +497,7 @@ UDTSTATUS CUDTUnited::getStatus(const UDTSOCKET u)
    return i->second->m_Status;   
 }
 
-int CUDTUnited::bind(const UDTSOCKET u, const sockaddr* name, const int& namelen)
+int CUDTUnited::bind(const UDTSOCKET u, const sockaddr* name, int namelen)
 {
    CUDTSocket* s = locate(u);
    if (NULL == s)
@@ -571,7 +572,7 @@ int CUDTUnited::bind(UDTSOCKET u, UDPSOCKET udpsock)
    return 0;
 }
 
-int CUDTUnited::listen(const UDTSOCKET u, const int& backlog)
+int CUDTUnited::listen(const UDTSOCKET u, int backlog)
 {
    CUDTSocket* s = locate(u);
    if (NULL == s)
@@ -642,7 +643,12 @@ UDTSOCKET CUDTUnited::accept(const UDTSOCKET listen, sockaddr* addr, int* addrle
       {
          pthread_mutex_lock(&(ls->m_AcceptLock));
 
-         if (ls->m_pQueuedSockets->size() > 0)
+         if ((LISTENING != ls->m_Status) || ls->m_pUDT->m_bBroken)
+         {
+            // This socket has been closed.
+            accepted = true;
+         }
+         else if (ls->m_pQueuedSockets->size() > 0)
          {
             u = *(ls->m_pQueuedSockets->begin());
             ls->m_pAcceptSockets->insert(ls->m_pAcceptSockets->end(), u);
@@ -650,15 +656,15 @@ UDTSOCKET CUDTUnited::accept(const UDTSOCKET listen, sockaddr* addr, int* addrle
             accepted = true;
          }
          else if (!ls->m_pUDT->m_bSynRecving)
+         {
             accepted = true;
-         else if (LISTENING == ls->m_Status)
+         }
+
+         if (!accepted && (LISTENING == ls->m_Status))
             pthread_cond_wait(&(ls->m_AcceptCond), &(ls->m_AcceptLock));
 
-         if ((LISTENING != ls->m_Status) || ls->m_pUDT->m_bBroken)
-            accepted = true;
-
          if (ls->m_pQueuedSockets->empty())
-            m_EPoll.disable_read(listen, ls->m_pUDT->m_sPollID);
+            m_EPoll.update_events(listen, ls->m_pUDT->m_sPollID, UDT_EPOLL_IN, false);
 
          pthread_mutex_unlock(&(ls->m_AcceptLock));
       }
@@ -685,12 +691,13 @@ UDTSOCKET CUDTUnited::accept(const UDTSOCKET listen, sockaddr* addr, int* addrle
 
          if ((LISTENING != ls->m_Status) || ls->m_pUDT->m_bBroken)
          {
+            // Send signal to other threads that are waiting to accept.
             SetEvent(ls->m_AcceptCond);
             accepted = true;
          }
 
          if (ls->m_pQueuedSockets->empty())
-            m_EPoll.disable_read(listen, ls->m_pUDT->m_sPollID);
+            m_EPoll.update_events(listen, ls->m_pUDT->m_sPollID, UDT_EPOLL_IN, false);
       }
    #endif
 
@@ -718,7 +725,7 @@ UDTSOCKET CUDTUnited::accept(const UDTSOCKET listen, sockaddr* addr, int* addrle
    return u;
 }
 
-int CUDTUnited::connect(const UDTSOCKET u, const sockaddr* name, const int& namelen)
+int CUDTUnited::connect(const UDTSOCKET u, const sockaddr* name, int namelen)
 {
    CUDTSocket* s = locate(u);
    if (NULL == s)
@@ -1083,8 +1090,10 @@ int CUDTUnited::epoll_create()
 int CUDTUnited::epoll_add_usock(const int eid, const UDTSOCKET u, const int* events)
 {
    CUDTSocket* s = locate(u);
+   int ret = -1;
    if (NULL != s)
    {
+      ret = m_EPoll.add_usock(eid, u, events);
       s->m_pUDT->addEPoll(eid);
    }
    else
@@ -1092,7 +1101,7 @@ int CUDTUnited::epoll_add_usock(const int eid, const UDTSOCKET u, const int* eve
       throw CUDTException(5, 4);
    }
 
-   return m_EPoll.add_usock(eid, u, events);
+   return ret;
 }
 
 int CUDTUnited::epoll_add_ssock(const int eid, const SYSSOCKET s, const int* events)
@@ -1100,8 +1109,10 @@ int CUDTUnited::epoll_add_ssock(const int eid, const SYSSOCKET s, const int* eve
    return m_EPoll.add_ssock(eid, s, events);
 }
 
-int CUDTUnited::epoll_remove_usock(const int eid, const UDTSOCKET u, const int* events)
+int CUDTUnited::epoll_remove_usock(const int eid, const UDTSOCKET u)
 {
+   int ret = m_EPoll.remove_usock(eid, u);
+
    CUDTSocket* s = locate(u);
    if (NULL != s)
    {
@@ -1112,12 +1123,12 @@ int CUDTUnited::epoll_remove_usock(const int eid, const UDTSOCKET u, const int*
    //   throw CUDTException(5, 4);
    //}
 
-   return m_EPoll.remove_usock(eid, u, events);
+   return ret;
 }
 
-int CUDTUnited::epoll_remove_ssock(const int eid, const SYSSOCKET s, const int* events)
+int CUDTUnited::epoll_remove_ssock(const int eid, const SYSSOCKET s)
 {
-   return m_EPoll.remove_ssock(eid, s, events);
+   return m_EPoll.remove_ssock(eid, s);
 }
 
 int CUDTUnited::epoll_wait(const int eid, set<UDTSOCKET>* readfds, set<UDTSOCKET>* writefds, int64_t msTimeOut, set<SYSSOCKET>* lrfds, set<SYSSOCKET>* lwfds)
@@ -1142,7 +1153,7 @@ CUDTSocket* CUDTUnited::locate(const UDTSOCKET u)
    return i->second;
 }
 
-CUDTSocket* CUDTUnited::locate(const sockaddr* peer, const UDTSOCKET& id, const int32_t& isn)
+CUDTSocket* CUDTUnited::locate(const sockaddr* peer, const UDTSOCKET id, int32_t isn)
 {
    CGuard cg(m_ControlLock);
 
@@ -1783,7 +1794,7 @@ int CUDT::send(UDTSOCKET u, const char* buf, int len, int)
    try
    {
       CUDT* udt = s_UDTUnited.lookup(u);
-      return udt->send((char*)buf, len);
+      return udt->send(buf, len);
    }
    catch (CUDTException e)
    {
@@ -1826,7 +1837,7 @@ int CUDT::sendmsg(UDTSOCKET u, const char* buf, int len, int ttl, bool inorder)
    try
    {
       CUDT* udt = s_UDTUnited.lookup(u);
-      return udt->sendmsg((char*)buf, len, ttl, inorder);
+      return udt->sendmsg(buf, len, ttl, inorder);
    }
    catch (CUDTException e)
    {
@@ -1864,7 +1875,7 @@ int CUDT::recvmsg(UDTSOCKET u, char* buf, int len)
    }
 }
 
-int64_t CUDT::sendfile(UDTSOCKET u, fstream& ifs, int64_t& offset, const int64_t& size, const int& block)
+int64_t CUDT::sendfile(UDTSOCKET u, fstream& ifs, int64_t& offset, int64_t size, int block)
 {
    try
    {
@@ -1888,7 +1899,7 @@ int64_t CUDT::sendfile(UDTSOCKET u, fstream& ifs, int64_t& offset, const int64_t
    }
 }
 
-int64_t CUDT::recvfile(UDTSOCKET u, fstream& ofs, int64_t& offset, const int64_t& size, const int& block)
+int64_t CUDT::recvfile(UDTSOCKET u, fstream& ofs, int64_t& offset, int64_t size, int block)
 {
    try
    {
@@ -2019,11 +2030,11 @@ int CUDT::epoll_add_ssock(const int eid, const SYSSOCKET s, const int* events)
    }
 }
 
-int CUDT::epoll_remove_usock(const int eid, const UDTSOCKET u, const int* events)
+int CUDT::epoll_remove_usock(const int eid, const UDTSOCKET u)
 {
    try
    {
-      return s_UDTUnited.epoll_remove_usock(eid, u, events);
+      return s_UDTUnited.epoll_remove_usock(eid, u);
    }
    catch (CUDTException e)
    {
@@ -2037,11 +2048,11 @@ int CUDT::epoll_remove_usock(const int eid, const UDTSOCKET u, const int* events
    }
 }
 
-int CUDT::epoll_remove_ssock(const int eid, const SYSSOCKET s, const int* events)
+int CUDT::epoll_remove_ssock(const int eid, const SYSSOCKET s)
 {
    try
    {
-      return s_UDTUnited.epoll_remove_ssock(eid, s, events);
+      return s_UDTUnited.epoll_remove_ssock(eid, s);
    }
    catch (CUDTException e)
    {
@@ -2167,7 +2178,7 @@ int bind(UDTSOCKET u, const struct sockaddr* name, int namelen)
    return CUDT::bind(u, name, namelen);
 }
 
-int bind(UDTSOCKET u, UDPSOCKET udpsock)
+int bind2(UDTSOCKET u, UDPSOCKET udpsock)
 {
    return CUDT::bind(u, udpsock);
 }
@@ -2242,6 +2253,22 @@ int64_t recvfile(UDTSOCKET u, fstream& ofs, int64_t& offset, int64_t size, int b
    return CUDT::recvfile(u, ofs, offset, size, block);
 }
 
+int64_t sendfile2(UDTSOCKET u, const char* path, int64_t* offset, int64_t size, int block)
+{
+   fstream ifs(path, ios::binary | ios::in);
+   int64_t ret = CUDT::sendfile(u, ifs, *offset, size, block);
+   ifs.close();
+   return ret;
+}
+
+int64_t recvfile2(UDTSOCKET u, const char* path, int64_t* offset, int64_t size, int block)
+{
+   fstream ofs(path, ios::binary | ios::out);
+   int64_t ret = CUDT::recvfile(u, ofs, *offset, size, block);
+   ofs.close();
+   return ret;
+}
+
 int select(int nfds, UDSET* readfds, UDSET* writefds, UDSET* exceptfds, const struct timeval* timeout)
 {
    return CUDT::select(nfds, readfds, writefds, exceptfds, timeout);
@@ -2257,32 +2284,82 @@ int epoll_create()
    return CUDT::epoll_create();
 }
 
-int epoll_add_usock(const int eid, const UDTSOCKET u, const int* events)
+int epoll_add_usock(int eid, UDTSOCKET u, const int* events)
 {
    return CUDT::epoll_add_usock(eid, u, events);
 }
 
-int epoll_add_ssock(const int eid, const SYSSOCKET s, const int* events)
+int epoll_add_ssock(int eid, SYSSOCKET s, const int* events)
 {
    return CUDT::epoll_add_ssock(eid, s, events);
 }
 
-int epoll_remove_usock(const int eid, const UDTSOCKET u, const int* events)
+int epoll_remove_usock(int eid, UDTSOCKET u)
 {
-   return CUDT::epoll_remove_usock(eid, u, events);
+   return CUDT::epoll_remove_usock(eid, u);
 }
 
-int epoll_remove_ssock(const int eid, const SYSSOCKET s, const int* events)
+int epoll_remove_ssock(int eid, SYSSOCKET s)
 {
-   return CUDT::epoll_remove_usock(eid, s, events);
+   return CUDT::epoll_remove_ssock(eid, s);
 }
 
-int epoll_wait(const int eid, set<int>* readfds, set<int>* writefds, int64_t msTimeOut, set<SYSSOCKET>* lrfds, set<SYSSOCKET>* lwfds)
+int epoll_wait(int eid, set<UDTSOCKET>* readfds, set<UDTSOCKET>* writefds, int64_t msTimeOut, set<SYSSOCKET>* lrfds, set<SYSSOCKET>* lwfds)
 {
    return CUDT::epoll_wait(eid, readfds, writefds, msTimeOut, lrfds, lwfds);
 }
 
-int epoll_release(const int eid)
+#define SET_RESULT(val, num, fds, it) \
+   if ((val != NULL) && !val->empty()) \
+   { \
+      if (*num > static_cast<int>(val->size())) \
+         *num = val->size(); \
+      int count = 0; \
+      for (it = val->begin(); it != val->end(); ++ it) \
+      { \
+         if (count >= *num) \
+            break; \
+         fds[count ++] = *it; \
+      } \
+   }
+int epoll_wait2(int eid, UDTSOCKET* readfds, int* rnum, UDTSOCKET* writefds, int* wnum, int64_t msTimeOut,
+                SYSSOCKET* lrfds, int* lrnum, SYSSOCKET* lwfds, int* lwnum)
+{
+   // This API is an alternative format for epoll_wait, created for compatability with other languages.
+   // Users need to pass in an array for holding the returned sockets, with the maximum array length
+   // stored in *rnum, etc., which will be updated with returned number of sockets.
+
+   set<UDTSOCKET> readset;
+   set<UDTSOCKET> writeset;
+   set<SYSSOCKET> lrset;
+   set<SYSSOCKET> lwset;
+   set<UDTSOCKET>* rval = NULL;
+   set<UDTSOCKET>* wval = NULL;
+   set<SYSSOCKET>* lrval = NULL;
+   set<SYSSOCKET>* lwval = NULL;
+   if ((readfds != NULL) && (rnum != NULL))
+      rval = &readset;
+   if ((writefds != NULL) && (wnum != NULL))
+      wval = &writeset;
+   if ((lrfds != NULL) && (lrnum != NULL))
+      lrval = &lrset;
+   if ((lwfds != NULL) && (lwnum != NULL))
+      lwval = &lwset;
+
+   int ret = CUDT::epoll_wait(eid, rval, wval, msTimeOut, lrval, lwval);
+   if (ret > 0)
+   {
+      set<UDTSOCKET>::const_iterator i;
+      SET_RESULT(rval, rnum, readfds, i);
+      SET_RESULT(wval, wnum, writefds, i);
+      set<SYSSOCKET>::const_iterator j;
+      SET_RESULT(lrval, lrnum, lrfds, j);
+      SET_RESULT(lwval, lwnum, lwfds, j);
+   }
+   return ret;
+}
+
+int epoll_release(int eid)
 {
    return CUDT::epoll_release(eid);
 }
@@ -2292,6 +2369,16 @@ ERRORINFO& getlasterror()
    return CUDT::getlasterror();
 }
 
+int getlasterror_code()
+{
+   return CUDT::getlasterror().getErrorCode();
+}
+
+const char* getlasterror_desc()
+{
+   return CUDT::getlasterror().getErrorMessage();
+}
+
 int perfmon(UDTSOCKET u, TRACEINFO* perf, bool clear)
 {
    return CUDT::perfmon(u, perf, clear);
@@ -2302,4 +2389,4 @@ UDTSTATUS getsockstate(UDTSOCKET u)
    return CUDT::getsockstate(u);
 }
 
-}
+}  // namespace UDT
