@@ -130,7 +130,7 @@ public:
       // Returned value:
       //    The new UDT socket ID, or INVALID_SOCK.
 
-   UDTSOCKET newSocket(const int& af, const int& type);
+   UDTSOCKET newSocket(int af, int type);
 
       // Functionality:
       //    Create a new UDT connection.
@@ -163,11 +163,11 @@ public:
 
       // socket APIs
 
-   int bind(const UDTSOCKET u, const sockaddr* name, const int& namelen);
+   int bind(const UDTSOCKET u, const sockaddr* name, int namelen);
    int bind(const UDTSOCKET u, UDPSOCKET udpsock);
-   int listen(const UDTSOCKET u, const int& backlog);
+   int listen(const UDTSOCKET u, int backlog);
    UDTSOCKET accept(const UDTSOCKET listen, sockaddr* addr, int* addrlen);
-   int connect(const UDTSOCKET u, const sockaddr* name, const int& namelen);
+   int connect(const UDTSOCKET u, const sockaddr* name, int namelen);
    int close(const UDTSOCKET u);
    int getpeername(const UDTSOCKET u, sockaddr* name, int* namelen);
    int getsockname(const UDTSOCKET u, sockaddr* name, int* namelen);
@@ -176,8 +176,8 @@ public:
    int epoll_create();
    int epoll_add_usock(const int eid, const UDTSOCKET u, const int* events = NULL);
    int epoll_add_ssock(const int eid, const SYSSOCKET s, const int* events = NULL);
-   int epoll_remove_usock(const int eid, const UDTSOCKET u, const int* events = NULL);
-   int epoll_remove_ssock(const int eid, const SYSSOCKET s, const int* events = NULL);
+   int epoll_remove_usock(const int eid, const UDTSOCKET u);
+   int epoll_remove_ssock(const int eid, const SYSSOCKET s);
    int epoll_wait(const int eid, std::set<UDTSOCKET>* readfds, std::set<UDTSOCKET>* writefds, int64_t msTimeOut, std::set<SYSSOCKET>* lrfds = NULL, std::set<SYSSOCKET>* lwfds = NULL);
    int epoll_release(const int eid);
 
@@ -199,6 +199,9 @@ public:
 
    CUDTException* getError();
 
+private:
+//   void init();
+
 private:
    std::map<UDTSOCKET, CUDTSocket*> m_Sockets;       // stores all the socket structures
 
@@ -222,7 +225,7 @@ private:
 private:
    void connect_complete(const UDTSOCKET u);
    CUDTSocket* locate(const UDTSOCKET u);
-   CUDTSocket* locate(const sockaddr* peer, const UDTSOCKET& id, const int32_t& isn);
+   CUDTSocket* locate(const sockaddr* peer, const UDTSOCKET id, int32_t isn);
    void updateMux(CUDTSocket* s, const sockaddr* addr = NULL, const UDPSOCKET* = NULL);
    void updateMux(CUDTSocket* s, const CUDTSocket* ls);
 
