@@ -35,7 +35,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 /*****************************************************************************
 written by
-   Yunhong Gu, last updated 05/07/2011
+   Yunhong Gu, last updated 02/28/2012
 *****************************************************************************/
 
 #ifndef WIN32
@@ -206,7 +206,7 @@ CUDT::~CUDT()
    delete m_pRNode;
 }
 
-void CUDT::setOpt(UDTOpt optName, const void* optval, const int&)
+void CUDT::setOpt(UDTOpt optName, const void* optval, int)
 {
    if (m_bBroken || m_bClosing)
       throw CUDTException(2, 1, 0);
@@ -343,8 +343,6 @@ void CUDT::setOpt(UDTOpt optName, const void* optval, const int&)
       break;
 
    case UDT_MAXBW:
-      if (m_bConnecting || m_bConnected)
-         throw CUDTException(5, 1, 0);
       m_llMaxBW = *(int64_t*)optval;
       break;
     
@@ -452,9 +450,9 @@ void CUDT::getOpt(UDTOpt optName, void* optval, int& optlen)
          event |= UDT_EPOLL_ERR;
       else
       {
-         if (m_pRcvBuffer->getRcvDataSize() > 0)
+         if (m_pRcvBuffer && (m_pRcvBuffer->getRcvDataSize() > 0))
             event |= UDT_EPOLL_IN;
-         if (m_iSndBufSize > m_pSndBuffer->getCurrBufSize())
+         if (m_pSndBuffer && (m_iSndBufSize > m_pSndBuffer->getCurrBufSize()))
             event |= UDT_EPOLL_OUT;
       }
       *(int32_t*)optval = event;
@@ -462,6 +460,22 @@ void CUDT::getOpt(UDTOpt optName, void* optval, int& optlen)
       break;
    }
 
+   case UDT_SNDDATA:
+      if (m_pSndBuffer)
+         *(int32_t*)optval = m_pSndBuffer->getCurrBufSize();
+      else
+         *(int32_t*)optval = 0;
+      optlen = sizeof(int32_t);
+      break;
+
+   case UDT_RCVDATA:
+      if (m_pRcvBuffer)
+         *(int32_t*)optval = m_pRcvBuffer->getRcvDataSize();
+      else
+         *(int32_t*)optval = 0;
+      optlen = sizeof(int32_t);
+      break;
+
    default:
       throw CUDTException(5, 0, 0);
    }
@@ -515,15 +529,12 @@ void CUDT::open()
 
    m_ullACKInt = m_ullSYNInt;
    m_ullNAKInt = m_ullMinNakInt;
-   m_ullEXPInt = m_ullMinExpInt;
-   m_llLastRspTime = CTimer::getTime();
 
-   CTimer::rdtsc(m_ullNextACKTime);
-   m_ullNextACKTime += m_ullSYNInt;
-   CTimer::rdtsc(m_ullNextNAKTime);
-   m_ullNextNAKTime += m_ullNAKInt;
-   CTimer::rdtsc(m_ullNextEXPTime);
-   m_ullNextEXPTime += m_ullEXPInt;
+   uint64_t currtime;
+   CTimer::rdtsc(currtime);
+   m_ullLastRspTime = currtime;
+   m_ullNextACKTime = currtime + m_ullSYNInt;
+   m_ullNextNAKTime = currtime + m_ullNAKInt;
 
    m_iPktCount = 0;
    m_iLightACKCount = 1;
@@ -772,13 +783,11 @@ POST_CONNECT:
    m_pCC = m_pCCFactory->create();
    m_pCC->m_UDT = m_SocketID;
    m_pCC->setMSS(m_iMSS);
-   m_pCC->setMaxCWndSize((int&)m_iFlowWindowSize);
-   m_pCC->setSndCurrSeqNo((int32_t&)m_iSndCurrSeqNo);
+   m_pCC->setMaxCWndSize(m_iFlowWindowSize);
+   m_pCC->setSndCurrSeqNo(m_iSndCurrSeqNo);
    m_pCC->setRcvRate(m_iDeliveryRate);
    m_pCC->setRTT(m_iRTT);
    m_pCC->setBandwidth(m_iBandwidth);
-   if (m_llMaxBW > 0)
-      m_pCC->setUserParam((char*)&(m_llMaxBW), 8);
    m_pCC->init();
 
    m_ullInterval = (uint64_t)(m_pCC->m_dPktSndPeriod * m_ullCPUFrequency);
@@ -792,12 +801,12 @@ POST_CONNECT:
    m_pRNode->m_bOnList = true;
    m_pRcvQueue->setNewEntry(this);
 
-   // acknowledde any waiting epolls to write
-   s_UDTUnited.m_EPoll.enable_write(m_SocketID, m_sPollID);
-
    // acknowledge the management module.
    s_UDTUnited.connect_complete(m_SocketID);
 
+   // acknowledde any waiting epolls to write
+   s_UDTUnited.m_EPoll.update_events(m_SocketID, m_sPollID, UDT_EPOLL_OUT, true);
+
    return 0;
 }
 
@@ -872,12 +881,11 @@ void CUDT::connect(const sockaddr* peer, CHandShake* hs)
    m_pCC = m_pCCFactory->create();
    m_pCC->m_UDT = m_SocketID;
    m_pCC->setMSS(m_iMSS);
-   m_pCC->setMaxCWndSize((int&)m_iFlowWindowSize);
-   m_pCC->setSndCurrSeqNo((int32_t&)m_iSndCurrSeqNo);
+   m_pCC->setMaxCWndSize(m_iFlowWindowSize);
+   m_pCC->setSndCurrSeqNo(m_iSndCurrSeqNo);
    m_pCC->setRcvRate(m_iDeliveryRate);
    m_pCC->setRTT(m_iRTT);
    m_pCC->setBandwidth(m_iBandwidth);
-   if (m_llMaxBW > 0) m_pCC->setUserParam((char*)&(m_llMaxBW), 8);
    m_pCC->init();
 
    m_ullInterval = (uint64_t)(m_pCC->m_dPktSndPeriod * m_ullCPUFrequency);
@@ -943,7 +951,9 @@ void CUDT::close()
    if (m_bConnected)
       m_pSndQueue->m_pSndUList->remove(this);
 
-   // remove itself from all epoll monitoring
+   // trigger any pending IO events.
+   s_UDTUnited.m_EPoll.update_events(m_SocketID, m_sPollID, UDT_EPOLL_ERR, true);
+   // then remove itself from all epoll monitoring
    try
    {
       for (set<int>::iterator i = m_sPollID.begin(); i != m_sPollID.end(); ++ i)
@@ -969,7 +979,7 @@ void CUDT::close()
       m_bListening = false;
       m_pRcvQueue->removeListener(this);
    }
-   else
+   else if (m_bConnecting)
    {
       m_pRcvQueue->removeConnector(m_SocketID);
    }
@@ -981,6 +991,7 @@ void CUDT::close()
 
       m_pCC->close();
 
+      // Store current connection information.
       CInfoBlock ib;
       ib.m_iIPversion = m_iIPversion;
       CInfoBlock::convert(m_pPeerAddr, m_iIPversion, ib.m_piIP);
@@ -999,7 +1010,7 @@ void CUDT::close()
    m_bOpened = false;
 }
 
-int CUDT::send(const char* data, const int& len)
+int CUDT::send(const char* data, int len)
 {
    if (UDT_DGRAM == m_iSockType)
       throw CUDTException(5, 10, 0);
@@ -1020,7 +1031,7 @@ int CUDT::send(const char* data, const int& len)
       // delay the EXP timer to avoid mis-fired timeout
       uint64_t currtime;
       CTimer::rdtsc(currtime);
-      m_ullNextEXPTime = currtime + m_ullEXPInt;
+      m_ullLastRspTime = currtime;
    }
 
    if (m_iSndBufSize <= m_pSndBuffer->getCurrBufSize())
@@ -1080,7 +1091,7 @@ int CUDT::send(const char* data, const int& len)
    if (m_iSndBufSize <= m_pSndBuffer->getCurrBufSize())
    {
       if (m_iSndTimeOut >= 0)
-         throw CUDTException(6, 1, 0); 
+         throw CUDTException(6, 3, 0); 
 
       return 0;
    }
@@ -1102,13 +1113,13 @@ int CUDT::send(const char* data, const int& len)
    if (m_iSndBufSize <= m_pSndBuffer->getCurrBufSize())
    {
       // write is not available any more
-      s_UDTUnited.m_EPoll.disable_write(m_SocketID, m_sPollID);
+      s_UDTUnited.m_EPoll.update_events(m_SocketID, m_sPollID, UDT_EPOLL_OUT, false);
    }
 
    return size;
 }
 
-int CUDT::recv(char* data, const int& len)
+int CUDT::recv(char* data, int len)
 {
    if (UDT_DGRAM == m_iSockType)
       throw CUDTException(5, 10, 0);
@@ -1186,16 +1197,16 @@ int CUDT::recv(char* data, const int& len)
    if (m_pRcvBuffer->getRcvDataSize() <= 0)
    {
       // read is not available any more
-      s_UDTUnited.m_EPoll.disable_read(m_SocketID, m_sPollID);
+      s_UDTUnited.m_EPoll.update_events(m_SocketID, m_sPollID, UDT_EPOLL_IN, false);
    }
 
    if ((res <= 0) && (m_iRcvTimeOut >= 0))
-      throw CUDTException(6, 2, 0);
+      throw CUDTException(6, 3, 0);
 
    return res;
 }
 
-int CUDT::sendmsg(const char* data, const int& len, const int& msttl, const bool& inorder)
+int CUDT::sendmsg(const char* data, int len, int msttl, bool inorder)
 {
    if (UDT_STREAM == m_iSockType)
       throw CUDTException(5, 9, 0);
@@ -1219,7 +1230,7 @@ int CUDT::sendmsg(const char* data, const int& len, const int& msttl, const bool
       // delay the EXP timer to avoid mis-fired timeout
       uint64_t currtime;
       CTimer::rdtsc(currtime);
-      m_ullNextEXPTime = currtime + m_ullEXPInt;
+      m_ullLastRspTime = currtime;
    }
 
    if ((m_iSndBufSize - m_pSndBuffer->getCurrBufSize()) * m_iPayloadSize < len)
@@ -1274,7 +1285,7 @@ int CUDT::sendmsg(const char* data, const int& len, const int& msttl, const bool
    if ((m_iSndBufSize - m_pSndBuffer->getCurrBufSize()) * m_iPayloadSize < len)
    {
       if (m_iSndTimeOut >= 0)
-         throw CUDTException(6, 1, 0);
+         throw CUDTException(6, 3, 0);
 
       return 0;
    }
@@ -1292,13 +1303,13 @@ int CUDT::sendmsg(const char* data, const int& len, const int& msttl, const bool
    if (m_iSndBufSize <= m_pSndBuffer->getCurrBufSize())
    {
       // write is not available any more
-      s_UDTUnited.m_EPoll.disable_write(m_SocketID, m_sPollID);
+      s_UDTUnited.m_EPoll.update_events(m_SocketID, m_sPollID, UDT_EPOLL_OUT, false);
    }
 
    return len;   
 }
 
-int CUDT::recvmsg(char* data, const int& len)
+int CUDT::recvmsg(char* data, int len)
 {
    if (UDT_STREAM == m_iSockType)
       throw CUDTException(5, 9, 0);
@@ -1319,7 +1330,7 @@ int CUDT::recvmsg(char* data, const int& len)
       if (m_pRcvBuffer->getRcvMsgNum() <= 0)
       {
          // read is not available any more
-         s_UDTUnited.m_EPoll.disable_read(m_SocketID, m_sPollID);
+         s_UDTUnited.m_EPoll.update_events(m_SocketID, m_sPollID, UDT_EPOLL_IN, false);
       }
 
       if (0 == res)
@@ -1388,16 +1399,16 @@ int CUDT::recvmsg(char* data, const int& len)
    if (m_pRcvBuffer->getRcvMsgNum() <= 0)
    {
       // read is not available any more
-      s_UDTUnited.m_EPoll.disable_read(m_SocketID, m_sPollID);
+      s_UDTUnited.m_EPoll.update_events(m_SocketID, m_sPollID, UDT_EPOLL_IN, false);
    }
 
    if ((res <= 0) && (m_iRcvTimeOut >= 0))
-      throw CUDTException(6, 2, 0);
+      throw CUDTException(6, 3, 0);
 
    return res;
 }
 
-int64_t CUDT::sendfile(fstream& ifs, int64_t& offset, const int64_t& size, const int& block)
+int64_t CUDT::sendfile(fstream& ifs, int64_t& offset, int64_t size, int block)
 {
    if (UDT_DGRAM == m_iSockType)
       throw CUDTException(5, 10, 0);
@@ -1417,7 +1428,7 @@ int64_t CUDT::sendfile(fstream& ifs, int64_t& offset, const int64_t& size, const
       // delay the EXP timer to avoid mis-fired timeout
       uint64_t currtime;
       CTimer::rdtsc(currtime);
-      m_ullNextEXPTime = currtime + m_ullEXPInt;
+      m_ullLastRspTime = currtime;
    }
 
    int64_t tosend = size;
@@ -1484,13 +1495,13 @@ int64_t CUDT::sendfile(fstream& ifs, int64_t& offset, const int64_t& size, const
    if (m_iSndBufSize <= m_pSndBuffer->getCurrBufSize())
    {
       // write is not available any more
-      s_UDTUnited.m_EPoll.disable_write(m_SocketID, m_sPollID);
+      s_UDTUnited.m_EPoll.update_events(m_SocketID, m_sPollID, UDT_EPOLL_OUT, false);
    }
 
    return size - tosend;
 }
 
-int64_t CUDT::recvfile(fstream& ofs, int64_t& offset, const int64_t& size, const int& block)
+int64_t CUDT::recvfile(fstream& ofs, int64_t& offset, int64_t size, int block)
 {
    if (UDT_DGRAM == m_iSockType)
       throw CUDTException(5, 10, 0);
@@ -1559,7 +1570,7 @@ int64_t CUDT::recvfile(fstream& ofs, int64_t& offset, const int64_t& size, const
    if (m_pRcvBuffer->getRcvDataSize() <= 0)
    {
       // read is not available any more
-      s_UDTUnited.m_EPoll.disable_read(m_SocketID, m_sPollID);
+      s_UDTUnited.m_EPoll.update_events(m_SocketID, m_sPollID, UDT_EPOLL_IN, false);
    }
 
    return size - torecv;
@@ -1605,7 +1616,7 @@ void CUDT::sample(CPerfMon* perf, bool clear)
    perf->usPktSndPeriod = m_ullInterval / double(m_ullCPUFrequency);
    perf->pktFlowWindow = m_iFlowWindowSize;
    perf->pktCongestionWindow = (int)m_dCongestionWindow;
-   perf->pktFlightSize = CSeqNo::seqlen(const_cast<int32_t&>(m_iSndLastAck), CSeqNo::incseq(m_iSndCurrSeqNo)) - 1;
+   perf->pktFlightSize = CSeqNo::seqlen(m_iSndLastAck, CSeqNo::incseq(m_iSndCurrSeqNo)) - 1;
    perf->msRTT = m_iRTT/1000.0;
    perf->mbpsBandwidth = m_iBandwidth * m_iPayloadSize * 8.0 / 1000000.0;
 
@@ -1638,6 +1649,18 @@ void CUDT::sample(CPerfMon* perf, bool clear)
    }
 }
 
+void CUDT::CCUpdate()
+{
+   m_ullInterval = (uint64_t)(m_pCC->m_dPktSndPeriod * m_ullCPUFrequency);
+   m_dCongestionWindow = m_pCC->m_dCWndSize;
+
+   if (m_llMaxBW <= 0)
+      return;
+   const double minSP = 1000000.0 / (double(m_llMaxBW) / m_iMSS) * m_ullCPUFrequency;
+   if (m_ullInterval < minSP)
+       m_ullInterval = minSP;
+}
+
 void CUDT::initSynch()
 {
    #ifndef WIN32
@@ -1711,7 +1734,7 @@ void CUDT::releaseSynch()
    #endif
 }
 
-void CUDT::sendCtrl(const int& pkttype, void* lparam, void* rparam, const int& size)
+void CUDT::sendCtrl(int pkttype, void* lparam, void* rparam, int size)
 {
    CPacket ctrlpkt;
 
@@ -1766,7 +1789,7 @@ void CUDT::sendCtrl(const int& pkttype, void* lparam, void* rparam, const int& s
          #endif
 
          // acknowledge any waiting epolls to read
-         s_UDTUnited.m_EPoll.enable_read(m_SocketID, m_sPollID);
+         s_UDTUnited.m_EPoll.update_events(m_SocketID, m_sPollID, UDT_EPOLL_IN, true);
       }
       else if (ack == m_iRcvLastAck)
       {
@@ -1932,17 +1955,9 @@ void CUDT::processCtrl(CPacket& ctrlpkt)
 {
    // Just heard from the peer, reset the expiration count.
    m_iEXPCount = 1;
-   m_llLastRspTime = CTimer::getTime();
-
-   if ((CSeqNo::incseq(m_iSndCurrSeqNo) == m_iSndLastAck) || (2 == ctrlpkt.getType()) || (3 == ctrlpkt.getType()))
-   {
-      uint64_t currtime;
-      CTimer::rdtsc(currtime);
-      if (!m_pCC->m_bUserDefinedRTO)
-         m_ullNextEXPTime = currtime + m_ullEXPInt;
-      else
-         m_ullNextEXPTime = currtime + m_pCC->m_iRTO * m_ullCPUFrequency;
-   }
+   uint64_t currtime;
+   CTimer::rdtsc(currtime);
+   m_ullLastRspTime = currtime;
 
    switch (ctrlpkt.getType())
    {
@@ -1954,9 +1969,9 @@ void CUDT::processCtrl(CPacket& ctrlpkt)
       if (4 == ctrlpkt.getLength())
       {
          ack = *(int32_t *)ctrlpkt.m_pcData;
-         if (CSeqNo::seqcmp(ack, const_cast<int32_t&>(m_iSndLastAck)) >= 0)
+         if (CSeqNo::seqcmp(ack, m_iSndLastAck) >= 0)
          {
-            m_iFlowWindowSize -= CSeqNo::seqoff(const_cast<int32_t&>(m_iSndLastAck), ack);
+            m_iFlowWindowSize -= CSeqNo::seqoff(m_iSndLastAck, ack);
             m_iSndLastAck = ack;
          }
 
@@ -1968,12 +1983,12 @@ void CUDT::processCtrl(CPacket& ctrlpkt)
 
       // send ACK acknowledgement
       // number of ACK2 can be much less than number of ACK
-      uint64_t currtime = CTimer::getTime();
+      uint64_t now = CTimer::getTime();
       if ((currtime - m_ullSndLastAck2Time > (uint64_t)m_iSYNInterval) || (ack == m_iSndLastAck2))
       {
          sendCtrl(6, &ack);
          m_iSndLastAck2 = ack;
-         m_ullSndLastAck2Time = currtime;
+         m_ullSndLastAck2Time = now;
       }
 
       // Got data ACK
@@ -1988,7 +2003,7 @@ void CUDT::processCtrl(CPacket& ctrlpkt)
          break;
       }
 
-      if (CSeqNo::seqcmp(ack, const_cast<int32_t&>(m_iSndLastAck)) >= 0)
+      if (CSeqNo::seqcmp(ack, m_iSndLastAck) >= 0)
       {
          // Update Flow Window Size, must update before and together with m_iSndLastAck
          m_iFlowWindowSize = *((int32_t *)ctrlpkt.m_pcData + 3);
@@ -1998,7 +2013,7 @@ void CUDT::processCtrl(CPacket& ctrlpkt)
       // protect packet retransmission
       CGuard::enterCS(m_AckLock);
 
-      int offset = CSeqNo::seqoff((int32_t&)m_iSndLastDataAck, ack);
+      int offset = CSeqNo::seqoff(m_iSndLastDataAck, ack);
       if (offset <= 0)
       {
          // discard it if it is a repeated ACK
@@ -2016,7 +2031,7 @@ void CUDT::processCtrl(CPacket& ctrlpkt)
 
       // update sending variables
       m_iSndLastDataAck = ack;
-      m_pSndLossList->remove(CSeqNo::decseq((int32_t&)m_iSndLastDataAck));
+      m_pSndLossList->remove(CSeqNo::decseq(m_iSndLastDataAck));
 
       CGuard::leaveCS(m_AckLock);
 
@@ -2031,7 +2046,7 @@ void CUDT::processCtrl(CPacket& ctrlpkt)
       #endif
 
       // acknowledde any waiting epolls to write
-      s_UDTUnited.m_EPoll.enable_write(m_SocketID, m_sPollID);
+      s_UDTUnited.m_EPoll.update_events(m_SocketID, m_sPollID, UDT_EPOLL_OUT, true);
 
       // insert this socket to snd list if it is not on the list yet
       m_pSndQueue->m_pSndUList->update(this, false);
@@ -2045,10 +2060,6 @@ void CUDT::processCtrl(CPacket& ctrlpkt)
 
       m_pCC->setRTT(m_iRTT);
 
-      m_ullEXPInt = (m_iRTT + 4 * m_iRTTVar) * m_ullCPUFrequency + m_ullSYNInt;
-      if (m_ullEXPInt < m_ullMinExpInt)
-         m_ullEXPInt = m_ullMinExpInt;
-
       if (ctrlpkt.getLength() > 16)
       {
          // Update Estimated Bandwidth and packet delivery rate
@@ -2063,9 +2074,7 @@ void CUDT::processCtrl(CPacket& ctrlpkt)
       }
 
       m_pCC->onACK(ack);
-      // update CC parameters
-      m_ullInterval = (uint64_t)(m_pCC->m_dPktSndPeriod * m_ullCPUFrequency);
-      m_dCongestionWindow = m_pCC->m_dCWndSize;
+      CCUpdate();
 
       ++ m_iRecvACK;
       ++ m_iRecvACKTotal;
@@ -2092,10 +2101,6 @@ void CUDT::processCtrl(CPacket& ctrlpkt)
 
       m_pCC->setRTT(m_iRTT);
 
-      m_ullEXPInt = (m_iRTT + 4 * m_iRTTVar) * m_ullCPUFrequency + m_ullSYNInt;
-      if (m_ullEXPInt < m_ullMinExpInt)
-         m_ullEXPInt = m_ullMinExpInt;
-
       // update last ACK that has been received by the sender
       if (CSeqNo::seqcmp(ack, m_iRcvLastAckAck) > 0)
          m_iRcvLastAckAck = ack;
@@ -2108,9 +2113,7 @@ void CUDT::processCtrl(CPacket& ctrlpkt)
       int32_t* losslist = (int32_t *)(ctrlpkt.m_pcData);
 
       m_pCC->onLoss(losslist, ctrlpkt.getLength() / 4);
-      // update CC parameters
-      m_ullInterval = (uint64_t)(m_pCC->m_dPktSndPeriod * m_ullCPUFrequency);
-      m_dCongestionWindow = m_pCC->m_dCWndSize;
+      CCUpdate();
 
       bool secure = true;
 
@@ -2119,7 +2122,7 @@ void CUDT::processCtrl(CPacket& ctrlpkt)
       {
          if (0 != (losslist[i] & 0x80000000))
          {
-            if ((CSeqNo::seqcmp(losslist[i] & 0x7FFFFFFF, losslist[i + 1]) > 0) || (CSeqNo::seqcmp(losslist[i + 1], const_cast<int32_t&>(m_iSndCurrSeqNo)) > 0))
+            if ((CSeqNo::seqcmp(losslist[i] & 0x7FFFFFFF, losslist[i + 1]) > 0) || (CSeqNo::seqcmp(losslist[i + 1], m_iSndCurrSeqNo) > 0))
             {
                // seq_a must not be greater than seq_b; seq_b must not be greater than the most recent sent seq
                secure = false;
@@ -2127,19 +2130,19 @@ void CUDT::processCtrl(CPacket& ctrlpkt)
             }
 
             int num = 0;
-            if (CSeqNo::seqcmp(losslist[i] & 0x7FFFFFFF, const_cast<int32_t&>(m_iSndLastAck)) >= 0)
+            if (CSeqNo::seqcmp(losslist[i] & 0x7FFFFFFF, m_iSndLastAck) >= 0)
                num = m_pSndLossList->insert(losslist[i] & 0x7FFFFFFF, losslist[i + 1]);
-            else if (CSeqNo::seqcmp(losslist[i + 1], const_cast<int32_t&>(m_iSndLastAck)) >= 0)
-               num = m_pSndLossList->insert(const_cast<int32_t&>(m_iSndLastAck), losslist[i + 1]);
+            else if (CSeqNo::seqcmp(losslist[i + 1], m_iSndLastAck) >= 0)
+               num = m_pSndLossList->insert(m_iSndLastAck, losslist[i + 1]);
 
             m_iTraceSndLoss += num;
             m_iSndLossTotal += num;
 
             ++ i;
          }
-         else if (CSeqNo::seqcmp(losslist[i], const_cast<int32_t&>(m_iSndLastAck)) >= 0)
+         else if (CSeqNo::seqcmp(losslist[i], m_iSndLastAck) >= 0)
          {
-            if (CSeqNo::seqcmp(losslist[i], const_cast<int32_t&>(m_iSndCurrSeqNo)) > 0)
+            if (CSeqNo::seqcmp(losslist[i], m_iSndCurrSeqNo) > 0)
             {
                //seq_a must not be greater than the most recent sent seq
                secure = false;
@@ -2248,9 +2251,7 @@ void CUDT::processCtrl(CPacket& ctrlpkt)
 
    case 32767: //0x7FFF - reserved and user defined messages
       m_pCC->processCustomMsg(&ctrlpkt);
-      // update CC parameters
-      m_ullInterval = (uint64_t)(m_pCC->m_dPktSndPeriod * m_ullCPUFrequency);
-      m_dCongestionWindow = m_pCC->m_dCWndSize;
+      CCUpdate();
 
       break;
 
@@ -2276,7 +2277,7 @@ int CUDT::packData(CPacket& packet, uint64_t& ts)
       // protect m_iSndLastDataAck from updating by ACK processing
       CGuard ackguard(m_AckLock);
 
-      int offset = CSeqNo::seqoff((int32_t&)m_iSndLastDataAck, packet.m_iSeqNo);
+      int offset = CSeqNo::seqoff(m_iSndLastDataAck, packet.m_iSeqNo);
       if (offset < 0)
          return 0;
 
@@ -2295,7 +2296,7 @@ int CUDT::packData(CPacket& packet, uint64_t& ts)
          m_pSndLossList->remove(seqpair[1]);
 
          // skip all dropped packets
-         if (CSeqNo::seqcmp(const_cast<int32_t&>(m_iSndCurrSeqNo), CSeqNo::incseq(seqpair[1])) < 0)
+         if (CSeqNo::seqcmp(m_iSndCurrSeqNo, CSeqNo::incseq(seqpair[1])) < 0)
              m_iSndCurrSeqNo = CSeqNo::incseq(seqpair[1]);
 
          return 0;
@@ -2312,12 +2313,12 @@ int CUDT::packData(CPacket& packet, uint64_t& ts)
 
       // check congestion/flow window limit
       int cwnd = (m_iFlowWindowSize < (int)m_dCongestionWindow) ? m_iFlowWindowSize : (int)m_dCongestionWindow;
-      if (cwnd >= CSeqNo::seqlen(const_cast<int32_t&>(m_iSndLastAck), CSeqNo::incseq(m_iSndCurrSeqNo)))
+      if (cwnd >= CSeqNo::seqlen(m_iSndLastAck, CSeqNo::incseq(m_iSndCurrSeqNo)))
       {
          if (0 != (payload = m_pSndBuffer->readData(&(packet.m_pcData), packet.m_iMsgNo)))
          {
             m_iSndCurrSeqNo = CSeqNo::incseq(m_iSndCurrSeqNo);
-            m_pCC->setSndCurrSeqNo((int32_t&)m_iSndCurrSeqNo);
+            m_pCC->setSndCurrSeqNo(m_iSndCurrSeqNo);
 
             packet.m_iSeqNo = m_iSndCurrSeqNo;
 
@@ -2387,22 +2388,12 @@ int CUDT::processData(CUnit* unit)
 
    // Just heard from the peer, reset the expiration count.
    m_iEXPCount = 1;
-   m_llLastRspTime = CTimer::getTime();
-
-   if (CSeqNo::incseq(m_iSndCurrSeqNo) == m_iSndLastAck)
-   {
-      uint64_t currtime;
-      CTimer::rdtsc(currtime);
-      if (!m_pCC->m_bUserDefinedRTO)
-         m_ullNextEXPTime = currtime + m_ullEXPInt;
-      else
-         m_ullNextEXPTime = currtime + m_pCC->m_iRTO * m_ullCPUFrequency;
-   }
+   uint64_t currtime;
+   CTimer::rdtsc(currtime);
+   m_ullLastRspTime = currtime;
 
    m_pCC->onPktReceived(&packet);
-
    ++ m_iPktCount;
-
    // update time information
    m_pRcvTimeWindow->onPktArrival();
 
@@ -2530,8 +2521,8 @@ int CUDT::listen(sockaddr* addr, CPacket& packet)
          }
          else
          {
-            // a mew connection has been created, enable epoll for write 
-            s_UDTUnited.m_EPoll.enable_write(m_SocketID, m_sPollID);
+            // a new connection has been created, enable epoll for write 
+            s_UDTUnited.m_EPoll.update_events(m_SocketID, m_sPollID, UDT_EPOLL_OUT, true);
          }
       }
    }
@@ -2542,8 +2533,7 @@ int CUDT::listen(sockaddr* addr, CPacket& packet)
 void CUDT::checkTimers()
 {
    // update CC parameters
-   m_ullInterval = (uint64_t)(m_pCC->m_dPktSndPeriod * m_ullCPUFrequency);
-   m_dCongestionWindow = m_pCC->m_dCWndSize;
+   CCUpdate();
    //uint64_t minint = (uint64_t)(m_ullCPUFrequency * m_pSndTimeWindow->getMinPktSndInt() * 0.9);
    //if (m_ullInterval < minint)
    //   m_ullInterval = minint;
@@ -2582,16 +2572,27 @@ void CUDT::checkTimers()
    //   m_ullNextNAKTime = currtime + m_ullNAKInt;
    //}
 
-   if (currtime > m_ullNextEXPTime)
+   uint64_t next_exp_time;
+   if (m_pCC->m_bUserDefinedRTO)
+      next_exp_time = m_ullLastRspTime + m_pCC->m_iRTO * m_ullCPUFrequency;
+   else
+   {
+      uint64_t exp_int = (m_iEXPCount * (m_iRTT + 4 * m_iRTTVar) + m_iSYNInterval) * m_ullCPUFrequency;
+      if (exp_int < m_iEXPCount * m_ullMinExpInt)
+         exp_int = m_iEXPCount * m_ullMinExpInt;
+      next_exp_time = m_ullLastRspTime + exp_int;
+   }
+
+   if (currtime > next_exp_time)
    {
       // Haven't receive any information from the peer, is it dead?!
       // timeout: at least 16 expirations and must be greater than 10 seconds
-      if ((m_iEXPCount > 16) && (CTimer::getTime() - m_llLastRspTime > 10000000))
+      if ((m_iEXPCount > 16) && (currtime - m_ullLastRspTime > 5000000 * m_ullCPUFrequency))
       {
          //
          // Connection is broken. 
          // UDT does not signal any information about this instead of to stop quietly.
-         // Apllication will detect this when it calls any UDT methods next time.
+         // Application will detect this when it calls any UDT methods next time.
          //
          m_bClosing = true;
          m_bBroken = true;
@@ -2603,8 +2604,7 @@ void CUDT::checkTimers()
          releaseSynch();
 
          // app can call any UDT API to learn the connection_broken error
-         s_UDTUnited.m_EPoll.enable_read(m_SocketID, m_sPollID);
-         s_UDTUnited.m_EPoll.enable_write(m_SocketID, m_sPollID);
+         s_UDTUnited.m_EPoll.update_events(m_SocketID, m_sPollID, UDT_EPOLL_IN | UDT_EPOLL_OUT | UDT_EPOLL_ERR, true);
 
          CTimer::triggerEvent();
 
@@ -2619,15 +2619,13 @@ void CUDT::checkTimers()
          {
             // resend all unacknowledged packets on timeout, but only if there is no packet in the loss list
             int32_t csn = m_iSndCurrSeqNo;
-            int num = m_pSndLossList->insert(const_cast<int32_t&>(m_iSndLastAck), csn);
+            int num = m_pSndLossList->insert(m_iSndLastAck, csn);
             m_iTraceSndLoss += num;
             m_iSndLossTotal += num;
          }
 
          m_pCC->onTimeout();
-         // update CC parameters
-         m_ullInterval = (uint64_t)(m_pCC->m_dPktSndPeriod * m_ullCPUFrequency);
-         m_dCongestionWindow = m_pCC->m_dCWndSize;
+         CCUpdate();
 
          // immediately restart transmission
          m_pSndQueue->m_pSndUList->update(this);
@@ -2638,11 +2636,8 @@ void CUDT::checkTimers()
       }
 
       ++ m_iEXPCount;
-      m_ullEXPInt = (m_iEXPCount * (m_iRTT + 4 * m_iRTTVar) + m_iSYNInterval) * m_ullCPUFrequency;
-      if (m_ullEXPInt < m_iEXPCount * m_ullMinExpInt)
-         m_ullEXPInt = m_iEXPCount * m_ullMinExpInt;
-      CTimer::rdtsc(m_ullNextEXPTime);
-      m_ullNextEXPTime += m_ullEXPInt;
+      // Reset last response time since we just sent a heart-beat.
+      m_ullLastRspTime = currtime;
    }
 }
 
@@ -2655,23 +2650,26 @@ void CUDT::addEPoll(const int eid)
    if (!m_bConnected || m_bBroken || m_bClosing)
       return;
 
-   if ((UDT_STREAM == m_iSockType) && (m_pRcvBuffer->getRcvDataSize() > 0))
-      s_UDTUnited.m_EPoll.enable_read(m_SocketID, m_sPollID);
-   else if ((UDT_DGRAM == m_iSockType) && (m_pRcvBuffer->getRcvMsgNum() > 0))
-      s_UDTUnited.m_EPoll.enable_read(m_SocketID, m_sPollID);
-
+   if (((UDT_STREAM == m_iSockType) && (m_pRcvBuffer->getRcvDataSize() > 0)) ||
+      ((UDT_DGRAM == m_iSockType) && (m_pRcvBuffer->getRcvMsgNum() > 0)))
+   {
+      s_UDTUnited.m_EPoll.update_events(m_SocketID, m_sPollID, UDT_EPOLL_IN, true);
+   }
    if (m_iSndBufSize > m_pSndBuffer->getCurrBufSize())
-      s_UDTUnited.m_EPoll.enable_write(m_SocketID, m_sPollID);
+   {
+      s_UDTUnited.m_EPoll.update_events(m_SocketID, m_sPollID, UDT_EPOLL_OUT, true);
+   }
 }
 
 void CUDT::removeEPoll(const int eid)
 {
+   // clear IO events notifications;
+   // since this happens after the epoll ID has been removed, they cannot be set again
+   set<int> remove;
+   remove.insert(eid);
+   s_UDTUnited.m_EPoll.update_events(m_SocketID, remove, UDT_EPOLL_IN | UDT_EPOLL_OUT, false);
+
    CGuard::enterCS(s_UDTUnited.m_EPoll.m_EPollLock);
    m_sPollID.erase(eid);
    CGuard::leaveCS(s_UDTUnited.m_EPoll.m_EPollLock);
-
-   // clear IO events notifications;
-   // since this happens after the epoll ID has been removed, they cannot be set again
-   s_UDTUnited.m_EPoll.disable_read(m_SocketID, m_sPollID);
-   s_UDTUnited.m_EPoll.disable_write(m_SocketID, m_sPollID);
 }
