@@ -1,5 +1,5 @@
 /*****************************************************************************
-Copyright (c) 2001 - 2009, The Board of Trustees of the University of Illinois.
+Copyright (c) 2001 - 2011, The Board of Trustees of the University of Illinois.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -35,12 +35,12 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 /*****************************************************************************
 written by
-   Yunhong Gu, last updated 05/05/2009
+   Yunhong Gu, last updated 01/22/2011
 *****************************************************************************/
 
 #include "list.h"
 
-CSndLossList::CSndLossList(const int& size):
+CSndLossList::CSndLossList(int size):
 m_piData1(NULL),
 m_piData2(NULL),
 m_piNext(NULL),
@@ -82,7 +82,7 @@ CSndLossList::~CSndLossList()
    #endif
 }
 
-int CSndLossList::insert(const int32_t& seqno1, const int32_t& seqno2)
+int CSndLossList::insert(int32_t seqno1, int32_t seqno2)
 {
    CGuard listguard(m_ListLock);
 
@@ -254,7 +254,7 @@ int CSndLossList::insert(const int32_t& seqno1, const int32_t& seqno2)
    return m_iLength - origlen;
 }
 
-void CSndLossList::remove(const int32_t& seqno)
+void CSndLossList::remove(int32_t seqno)
 {
    CGuard listguard(m_ListLock);
 
@@ -419,7 +419,7 @@ int32_t CSndLossList::getLostSeq()
 
 ////////////////////////////////////////////////////////////////////////////////
 
-CRcvLossList::CRcvLossList(const int& size):
+CRcvLossList::CRcvLossList(int size):
 m_piData1(NULL),
 m_piData2(NULL),
 m_piNext(NULL),
@@ -427,8 +427,7 @@ m_piPrior(NULL),
 m_iHead(-1),
 m_iTail(-1),
 m_iLength(0),
-m_iSize(size),
-m_TimeStamp()
+m_iSize(size)
 {
    m_piData1 = new int32_t [m_iSize];
    m_piData2 = new int32_t [m_iSize];
@@ -441,8 +440,6 @@ m_TimeStamp()
       m_piData1[i] = -1;
       m_piData2[i] = -1;
    }
-
-   m_TimeStamp = CTimer::getTime();
 }
 
 CRcvLossList::~CRcvLossList()
@@ -453,10 +450,8 @@ CRcvLossList::~CRcvLossList()
    delete [] m_piPrior;
 }
 
-void CRcvLossList::insert(const int32_t& seqno1, const int32_t& seqno2)
+void CRcvLossList::insert(int32_t seqno1, int32_t seqno2)
 {
-   m_TimeStamp = CTimer::getTime();
-
    // Data to be inserted must be larger than all those in the list
    // guaranteed by the UDT receiver
 
@@ -503,10 +498,8 @@ void CRcvLossList::insert(const int32_t& seqno1, const int32_t& seqno2)
    m_iLength += CSeqNo::seqlen(seqno1, seqno2);
 }
 
-bool CRcvLossList::remove(const int32_t& seqno)
+bool CRcvLossList::remove(int32_t seqno)
 {
-   m_TimeStamp = CTimer::getTime();
-
    if (0 == m_iLength)
       return false; 
 
@@ -635,7 +628,7 @@ bool CRcvLossList::remove(const int32_t& seqno)
    return true;
 }
 
-bool CRcvLossList::remove(const int32_t& seqno1, const int32_t& seqno2)
+bool CRcvLossList::remove(int32_t seqno1, int32_t seqno2)
 {
    if (seqno1 <= seqno2)
    {
@@ -653,7 +646,7 @@ bool CRcvLossList::remove(const int32_t& seqno1, const int32_t& seqno2)
    return true;
 }
 
-bool CRcvLossList::find(const int32_t& seqno1, const int32_t& seqno2) const
+bool CRcvLossList::find(int32_t seqno1, int32_t seqno2) const
 {
    if (0 == m_iLength)
       return false;
@@ -686,14 +679,10 @@ int CRcvLossList::getFirstLostSeq() const
    return m_piData1[m_iHead];
 }
 
-void CRcvLossList::getLossArray(int32_t* array, int& len, const int& limit, const int& threshold)
+void CRcvLossList::getLossArray(int32_t* array, int& len, int limit)
 {
    len = 0;
 
-   // do not feedback NAK unless no retransmission is received within a certain interval
-   if (int(CTimer::getTime() - m_TimeStamp) < threshold)
-      return;
-
    int i = m_iHead;
 
    while ((len < limit - 1) && (-1 != i))
@@ -711,6 +700,4 @@ void CRcvLossList::getLossArray(int32_t* array, int& len, const int& limit, cons
 
       i = m_piNext[i];
    }
-
-   m_TimeStamp = CTimer::getTime();
 }
